@@ -10,19 +10,19 @@
 #import "ESSIPEService.h"
 #import "ESSIPELibpurpleServicePlugin.h"
 
-#include "sipe.h"
+#include "sipe-core.h"
 
 extern void purple_init_sipe_plugin(void);
 
 @implementation ESSIPELibpurpleServicePlugin
 
-- (void)installLibpurplePlugin
+- (void)installLibpurplePlugin {}
+
+- (void)loadLibpurplePlugin 
 {
 	purple_init_sipe_plugin();
 }
 
-- (void)loadLibpurplePlugin {}
-
 - (void)installPlugin
 {
 	[super installPlugin];
