@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010-11 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-12 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -562,9 +562,12 @@ const gchar *sipe_backend_setting(struct sipe_core_public *sipe_public,
 /** STATUS *******************************************************************/
 
 guint sipe_backend_status(struct sipe_core_public *sipe_public);
-gboolean sipe_backend_status_and_note(struct sipe_core_public *sipe_public,
-				      guint activity,
-				      const gchar *message);
+gboolean sipe_backend_status_changed(struct sipe_core_public *sipe_public,
+				     guint activity,
+				     const gchar *message);
+void sipe_backend_status_and_note(struct sipe_core_public *sipe_public,
+				  guint activity,
+				  const gchar *message);
 
 /** TRANSPORT ****************************************************************/
 
